#include <stdexcept>
#include <string>
#include <sstream>
#include <unistd.h>
#include <sys/wait.h>
#include <libscerror.hpp>
#include <libscterm.hpp>
#include "pixels.hpp"

static std::string resize_str() {
    sc::term t {STDOUT_FILENO};
    int cols = t.cols() - 1;
    int rows = t.rows() * 2 - 4;
    std::ostringstream oss;
    oss << cols << "x" << rows;
    return oss.str();
}

void convert(const std::string& path) { // try to convert to ppm using ImageMagick
    int pfd[2];
    throw_if_min1_msg(pipe(pfd), "could not create pipe");

    switch (fork()) {
        case -1:
            throw std::runtime_error {"could not fork"};

        case 0: { // child
            std::string resize {resize_str()};

            throw_if_min1(dup2(pfd[1], STDOUT_FILENO));
            throw_if_min1(close(pfd[0]));
            throw_if_min1(close(pfd[1]));

            int res = execlp("magick", "magick", path.c_str(), "-resize", resize.c_str(), "ppm:-", nullptr);
            if (res == -1) { // probably "magick" not found, try "convert"
                res = execlp("convert", "convert", path.c_str(), "-resize", resize.c_str(), "ppm:-", nullptr);
                if (res == -1) { // no ImageMagick, abort
                    throw std::runtime_error {"no ImageMagick installed, abort"};
                }
            }
            break;
        }

        default: // parent
            throw_if_min1(dup2(pfd[0], STDIN_FILENO));
            throw_if_min1(close(pfd[0]));
            throw_if_min1(close(pfd[1]));

            write_image(std::cout, stdin);

            throw_if_min1(wait(nullptr));
            break;
    }
}
